// importing the properties from d3
import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  extent,
  max,
  axisLeft,
  axisBottom,
  area,
  curveBasis,
  format
} from 'd3';

const svg = select('svg');//to select svg element

// Add width and height of the area chart
const width = +svg.attr('width');
const height = +svg.attr('height');

// To display HTML code inside the specified HTML element.
const render = data => {
  const title = 'Taylor Swift Listening trends on Spotify in the year 2001';
  
 // labeling the attributes to the chart  
  const xValue = d => d.Date;
  const xAxisLabel = 'Month';//giving label to x-axis
  
  const yValue = d => d.Hours;
  const circleRadius = 5;
  const yAxisLabel = 'Hours of Listening';//giving label to y-axis
  
  const margin = { top: 60, right: 40, bottom: 88, left: 105 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  // Add scaleTime function to automatically adjust Months
  const xScale = scaleTime()
    .domain(extent(data, xValue))//returns the minimum and maximum value in an array from the given array using natural order.
    .range([0, innerWidth])//output bounds
    .nice();// it will round the values out
  
  // Add scaleLinear function to automatically adjust hours of listening
  const yScale = scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerHeight, 0])
    .nice();
  
  // Locate the graph
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  // Add attributes of x-axis
  const xAxis = axisBottom(xScale)
    .ticks(8)
    .tickSize(-innerHeight)
    .tickPadding(15);

  // Format y-axis tick
  const yAxisTickFormat = number =>
    format('.1s')(number)
      .replace('G', 'B');
  
    // Add attributes of y-axis
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(yAxisTickFormat);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  // Add attributes of y-axis
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -60)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  // Add attributes of x-axis
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 80)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  // Control the area of the area chart
  const areaGenerator = area()
    .x(d => xScale(xValue(d)))
    .y0(innerHeight)
    .y1(d => yScale(yValue(d)))
    .curve(curveBasis);
  
  // Add additributes to path
  g.append('path')
      .attr('class', 'line-path')
      .attr('d', areaGenerator(data));
  
  // for labeling the title
  svg.append('text')
      .attr('class', 'title')
      .attr('x', width / 2)
      .attr('y', 45)
      .text(title);
};

// getting the csv data uisng github url link for my (swift_trends.csv)data 
csv('https://gist.githubusercontent.com/Shivamaninaidu/8437ec09c24fa084630088b547570f7b/raw/95c633420631a445118f0ab105909068aafda3c7/fortuner.csv


')
  .then(data => {
    data.forEach(d => {
      d.Hours = +d.Hours;
      d.Date = new Date(d.Date);
    });
    render(data);
  });